# Generated automatically.  DO NOT EDIT!
NULL = 0
SCALAR = 1
TENSOR_DATA_SOURCE = 2
TENSOR_ONES = 3
TENSOR_ONES_LIKE = 4
TENSOR_ZEROS = 5
TENSOR_ZEROS_LIKE = 6
TENSOR_EMPTY = 7
TENSOR_EMPTY_LIKE = 8
TENSOR_FULL = 9
TENSOR_ARANGE = 10
TENSOR_INDICES = 11
TENSOR_DIAG = 12
TENSOR_EYE = 13
TENSOR_LINSPACE = 14
TENSOR_TRIU = 15
TENSOR_TRIL = 16
RAND_RAND = 41
RAND_RANDN = 42
RAND_RANDINT = 43
RAND_RANDOM_INTEGERS = 44
RAND_RANDOM_SAMPLE = 45
RAND_RANDOM = 46
RAND_RANF = 47
RAND_SAMPLE = 48
RAND_BYTES = 49
RAND_BETA = 50
RAND_BINOMIAL = 51
RAND_CHISQUARE = 52
RAND_CHOICE = 53
RAND_DIRICHLET = 54
RAND_EXPONENTIAL = 55
RAND_F = 56
RAND_GAMMA = 57
RAND_GEOMETRIC = 58
RAND_GUMBEL = 59
RAND_HYPERGEOMETRIC = 60
RAND_LAPLACE = 61
RAND_LOGISTIC = 62
RAND_LOGNORMAL = 63
RAND_LOGSERIES = 64
RAND_MULTINOMIAL = 65
RAND_MULTIVARIATE_NORMAL = 66
RAND_NEGATIVE_BINOMIAL = 67
RAND_NONCENTRAL_CHISQURE = 68
RAND_NONCENTRAL_F = 69
RAND_NORMAL = 70
RAND_PARETO = 71
RAND_PERMUTATION = 72
RAND_POSSION = 73
RAND_POWER = 74
RAND_RAYLEIGH = 75
RAND_SHUFFLE = 76
RAND_STANDARD_CAUCHY = 77
RAND_STANDARD_EXPONENTIAL = 78
RAND_STANDARD_GAMMMA = 79
RAND_STANDARD_NORMAL = 80
RAND_STANDARD_T = 81
RAND_TOMAXINT = 82
RAND_TRIANGULAR = 83
RAND_UNIFORM = 84
RAND_VONMISES = 85
RAND_WALD = 86
RAND_WEIBULL = 87
RAND_ZIPF = 88
ADD = 101
ADD_CONSTANT = 102
SUB = 103
SUB_CONSTANT = 104
MUL = 105
MUL_CONSTANT = 106
DIV = 107
DIV_CONSTANT = 108
TRUEDIV = 109
TDIV_CONSTANT = 110
FLOORDIV = 111
FDIV_CONSTANT = 112
POW = 113
POW_CONSTANT = 114
MOD = 115
MOD_CONSTANT = 116
FMOD = 117
FMOD_CONSTANT = 118
LOGADDEXP = 119
LAE_CONSTANT = 120
LOGADDEXP2 = 121
LAE2_CONSTANT = 122
NEGATIVE = 123
POSITIVE = 124
ABSOLUTE = 125
FABS = 126
ABS = 127
RINT = 128
SIGN = 129
CONJ = 130
EXP = 131
EXP2 = 132
LOG = 133
LOG2 = 134
LOG10 = 135
EXPM1 = 136
LOG1P = 137
SQRT = 138
SQUARE = 139
CBRT = 140
RECIPROCAL = 141
EQ = 142
EQ_CONSTANT = 143
NE = 144
NE_CONSTANT = 145
LT = 146
LT_CONSTANT = 147
LE = 148
LE_CONSTANT = 149
GT = 150
GT_CONSTANT = 151
GE = 152
GE_CONSTANT = 153
SIN = 154
COS = 155
TAN = 156
ARCSIN = 157
ARCCOS = 158
ARCTAN = 159
ARCTAN2 = 160
ARCT2_CONSTANT = 161
HYPOT = 162
HYPOT_CONSTANT = 163
SINH = 164
COSH = 165
TANH = 166
ARCSINH = 167
ARCCOSH = 168
ARCTANH = 169
DEG2RAD = 170
RAD2DEG = 171
BITAND = 172
BITAND_CONSTANT = 173
BITOR = 174
BITOR_CONSTANT = 175
BITXOR = 176
BITXOR_CONSTANT = 177
INVERT = 178
LSHIFT = 179
LSHIFT_CONSTANT = 180
RSHIFT = 181
RSHIFT_CONSTANT = 182
AND = 183
AND_CONSTANT = 184
OR = 185
OR_CONSTANT = 186
XOR = 187
XOR_CONSTANT = 188
NOT = 189
MAXIMUM = 190
MAXIMUM_CONSTANT = 191
MINIMUM = 192
MINIMUM_CONSTANT = 193
AROUND = 194
FLOAT_POWER = 195
FLOAT_POWER_CONSTANT = 196
FMAX = 197
FMAX_CONSTANT = 198
FMIN = 199
FMIN_CONSTANT = 200
ISFINITE = 201
ISINF = 202
ISNAN = 203
SIGNBIT = 204
COPYSIGN = 205
COPYSIGN_CONSTANT = 206
NEXTAFTER = 207
NEXTAFTER_CONSTANT = 208
SPACING = 209
LDEXP = 210
LDEXP_CONSTANT = 211
FREXP = 212
MODF = 213
FLOOR = 214
CEIL = 215
TRUNC = 216
DEGREES = 217
RADIANS = 218
CLIP = 219
ISREAL = 220
ISCOMPLEX = 221
REAL = 222
IMAG = 223
FIX = 224
I0 = 225
SINC = 226
NAN_TO_NUM = 227
ISCLOSE = 228
ISCLOSE_CONSTANT = 229
DIVMOD = 231
ANGLE = 232
SET_REAL = 233
SET_REAL_CONSTANT = 234
SET_IMAG = 235
SET_IMAG_CONSTANT = 236
TREE_ADD = 251
TREE_MULTIPLY = 252
CUMSUM = 301
CUMPROD = 302
PROD = 303
SUM = 304
MAX = 305
MIN = 306
ALL = 307
ANY = 308
MEAN_CHUNK = 309
MEAN_COMBINE = 310
MEAN = 311
ARGMAX = 312
ARGMAX_CHUNK = 313
ARGMAX_COMBINE = 314
ARGMIN = 315
ARGMIN_CHUNK = 316
ARGMIN_COMBINE = 317
NANSUM = 318
NANMAX = 319
NANMIN = 320
NANPROD = 321
NANMEAN = 322
NANMEAN_CHUNK = 323
NANARGMAX = 324
NANARGMAX_CHUNK = 325
NANARGMAX_COMBINE = 326
NANARGMIN = 327
NANARGMIN_CHUNK = 328
NANARGMIN_COMBINE = 329
COUNT_NONZERO = 330
MOMENT_CHUNK = 331
NANMOMENT_CHUNK = 332
MOMENT_COMBINE = 333
NANMOMENT_COMBINE = 334
MOMENT = 335
NANMOMENT = 336
VAR = 337
STD = 338
NANVAR = 339
NANSTD = 340
NANCUMSUM = 341
NANCUMPROD = 342
RESHAPE = 401
SLICE = 402
INDEX = 403
INDEXSETVALUE = 404
CONCATENATE = 405
RECHUNK = 406
ASTYPE = 407
TRANSPOSE = 408
SWAPAXES = 409
BROADCAST_TO = 410
STACK = 411
WHERE = 412
CHOOSE = 413
NONZERO = 414
ARGWHERE = 415
UNRAVEL_INDEX = 416
RAVEL_MULTI_INDEX = 417
ARRAY_SPLIT = 418
SQUEEZE = 419
DIGITIZE = 420
REPEAT = 421
COPYTO = 422
ISIN = 423
TENSORDOT = 501
DOT = 502
MATMUL = 503
CHOLESKY = 510
QR = 511
SVD = 512
LU = 513
SOLVE_TRIANGULAR = 520
NORM = 530
FFT = 601
IFFT = 602
FFT2 = 603
IFFT2 = 604
FFTN = 605
IFFTN = 606
RFFT = 607
IRFFT = 608
RFFT2 = 609
IRFFT2 = 610
RFFTN = 611
IRFFTN = 612
HFFT = 613
IHFFT = 614
FFTFREQ = 615
FFTFREQ_CHUNK = 616
RFFTFREQ = 617
FFTSHIFT = 618
IFFTSHIFT = 619
SPARSE_MATRIX_DATA_SOURCE = 701
DENSE_TO_SPARSE = 702
SPARSE_TO_DENSE = 703
FUSE = 801
ENTER = 901
LEAVE = 902
FIX_LATEST = 903
IF_ELSE = 904
NEXT_ITER = 905
TABLE_COO = 1003
STORE_COO = 1004
FETCH = 999999
